open Graph
open Gparser
open Glexer
open Printf

let lmax list =
  List.fold_left (fun x (a,b) -> max x (max a b)) 0 list 

let elist2ltab list =
  let n = (lmax list) + 1
  in let tab = Array.make n []
  in let add_edge (a,b) =
      if not (List.mem a tab.(b)) then
	begin
	  tab.(a) <- b :: tab.(a);
	  tab.(b) <- a :: tab.(b)
	end
  in 
    begin 
      List.iter add_edge list;
      tab
    end

let read c_in =
  let ltab = elist2ltab (edge_list enext_token (Lexing.from_channel c_in))
  in let n = Array.length ltab
  in let d = Array.fold_left (fun m l -> (max m (List.length l))) 0 ltab
  in let add_node l =
      let tab = Array.make d sharp
      and i = ref (0)
      in 
	begin 
	  List.iter (fun x -> tab.(!i) <- x; i := !i + 1) l;
	  tab
	end
  in Graph.make (Array.map add_node ltab)  


let file_read filename =
  let c_in = open_in filename
  in let g = raw_graph rnext_token (Lexing.from_channel c_in)
  in 
    close_in c_in;
    g 


let write graph c_out =
  begin
    fprintf c_out "%d %d\n" (size graph) (degree graph);
    Array.iteri
      (fun i nbs -> 
	 Array.iter 
	   (fun v -> 
	      if v >= 0 then fprintf c_out "%d " v
	      else fprintf c_out "# ")
	   nbs;
	 fprintf c_out "\n")
      (tab graph)
  end
    
	
let file_write graph filename =
  let c_out = open_out filename
  in 
    write graph c_out;
    close_out c_out
  
