
type graph = { tab : int array array;
	       mutable pos : (int * int) array
	     }

exception Found of int

let sharp = -1

external c_positions :
  int array array -> int -> int -> (int * int) array =
  "caml_positions" 

let graph_is_opened () = 
  try 
    Graphics.size_x ();
    true
  with Graphics.Graphic_failure str -> false 

let help () =
  let c_in = open_in "help.txt"
  in 
    begin
      try while true do print_endline (input_line c_in) done
      with End_of_file -> ()
    end;
    close_in c_in


let positions tab =
  if graph_is_opened () 
  then c_positions tab (Graphics.size_x ()) (Graphics.size_y ())
  else Array.make (Array.length tab) (0,0) 

let make tab = { tab = tab; pos = (positions tab) } 

let size graph = Array.length graph.tab 

let degree graph = Array.length graph.tab.(0) 

let tab graph = graph.tab 
let pos graph = graph.pos 

let recomputepos graph = graph.pos <- positions (tab graph) 

let nboors graph v =
  if (v < 0) || ((size graph) <= v) 
  then failwith "nboors: sommet en dehors du graphe."
  else (tab graph).(v) 

let is_nboor graph v1 v2 =
  if (v1 < 0) || (v2 < 0) || 
    ((size graph) <= v1) || ((size graph) <= v2) 
  then failwith "is_nboor: sommet en dehors du graphe."
  else
    try 
      Array.iter 
	(fun v -> if v = v2 then raise (Found 0))
	(nboors graph v1);
      false
    with
	Found i -> true 

let h_vector graph v =
  let pos x =
    try 
      Array.iteri
	(fun i y -> if x = y then raise (Found i))
	(nboors graph v);
      failwith "h_vector (pos): Bizarre..."
    with
	Found i -> i
  in Array.map pos (nboors graph v) 
